@@ -17,6 +17,8 @@ from benchmarks.user.tools import Driver, Executor, RooflinePlotter
 from examples.seismic.acoustic.acoustic_example import run as acoustic_run, acoustic_setup
 from examples.seismic.tti.tti_example import run as tti_run, tti_setup
 from examples.seismic.elastic.elastic_example import run as elastic_run, elastic_setup
+from examples.seismic.skew_self_adjoint.example_iso import run as acoustic_ssa_run, \
+    acoustic_ssa_setup
 from examples.seismic.viscoelastic.viscoelastic_example import run as viscoelastic_run, \
     viscoelastic_setup
 
@@ -41,10 +43,42 @@ model_type = {
         'run': acoustic_run,
         'setup': acoustic_setup,
         'default-section': 'section0'
+    },
+    'acoustic_ssa': {
+        'run': acoustic_ssa_run,
+        'setup': acoustic_ssa_setup,
+        'default-section': 'section0'
     }
 }
 
 
+def run_op(solver, operator, **options):
+    """
+    Initialize any necessary input and run the operator associated with the solver.
+    """
+    # Get the operator if exist
+    try:
+        op = getattr(solver, operator)
+    except AttributeError:
+        raise AttributeError("Operator %s not implemented for %s" % (operator, solver))
+
+    # This is a bit ugly but not sure how to make clean input creation for different op
+    if operator == "forward":
+        return op(**options)
+    elif operator == "adjoint":
+        rec = solver.geometry.adj_src
+        return op(rec, **options)
+    elif operator == "jacobian":
+        dm = solver.model.dm
+        return op(dm, **options)
+    elif operator == "jacobian_adjoint":
+        # I think we want the forward + gradient call, need to merge retvals
+        rec, u, _ = solver.forward(save=True, **options)
+        return op(rec, u, **options)
+    else:
+        raise ValueError("Unrecognized operator %s" % operator)
+
+
 @click.group()
 def benchmark():
     """
@@ -68,9 +102,9 @@ def option_simulation(f):
         return list(value if len(value) > 0 else (2, ))
 
     options = [
-        click.option('-P', '--problem', type=click.Choice(['acoustic', 'tti',
-                                                           'elastic', 'viscoelastic']),
-                     help='Problem name'),
+        click.option('-P', '--problem', help='Problem name',
+                     type=click.Choice(['acoustic', 'tti',
+                                        'elastic', 'acoustic_ssa', 'viscoelastic'])),
         click.option('-d', '--shape', default=(50, 50, 50),
                      help='Number of grid points along each axis'),
         click.option('-s', '--spacing', default=(20., 20., 20.),
@@ -82,8 +116,10 @@ def option_simulation(f):
         click.option('-to', '--time-order', type=int, multiple=True,
                      callback=default_list, help='Time order of the simulation'),
         click.option('-t', '--tn', default=250,
-                     help='End time of the simulation in ms')
-    ]
+                     help='End time of the simulation in ms'),
+        click.option('-op', '--operator', default='forward', help='Operator to run',
+                     type=click.Choice(['forward', 'adjoint',
+                                        'jacobian', 'jacobian_adjoint']))]
     for option in reversed(options):
         f = option(f)
     return f
@@ -203,7 +239,9 @@ def run(problem, **kwargs):
     time_order = kwargs.pop('time_order')[0]
     space_order = kwargs.pop('space_order')[0]
     autotune = kwargs.pop('autotune')
+    options['autotune'] = autotune
     block_shapes = as_tuple(kwargs.pop('block_shape'))
+    operator = kwargs.pop('operator', 'forward')
 
     # Should a specific block-shape be used? Useful if one wants to skip
     # the autotuning pass as a good block-shape is already known
@@ -214,21 +252,32 @@ def run(problem, **kwargs):
                 options['%s%d_blk%d_size' % (d, i, n)] = s
 
     solver = setup(space_order=space_order, time_order=time_order, **kwargs)
-    retval = solver.forward(autotune=autotune, **options)
+    retval = run_op(solver, operator, **options)
+
+    try:
+        rank = MPI.COMM_WORLD.rank
+    except AttributeError:
+        # MPI not available
+        rank = 0
 
     dumpfile = kwargs.pop('dump_summary')
     if dumpfile:
-        with open(dumpfile, 'w') as f:
-            summary = retval[-1]
-            assert isinstance(summary, PerformanceSummary)
-            f.write(str(summary.globals['fdlike']))
+        if configuration['profiling'] != 'advanced':
+            raise RuntimeError("Must set DEVITO_PROFILING=advanced (or, alternatively, "
+                               "DEVITO_LOGGING=PERF) with --dump-summary")
+        if rank == 0:
+            with open(dumpfile, 'w') as f:
+                summary = retval[-1]
+                assert isinstance(summary, PerformanceSummary)
+                f.write(str(summary.globals['fdlike']))
 
     dumpfile = kwargs.pop('dump_norms')
     if dumpfile:
         norms = ["'%s': %f" % (i.name, norm(i)) for i in retval[:-1]
                  if isinstance(i, DiscreteFunction)]
-        with open(dumpfile, 'w') as f:
-            f.write("{%s}" % ', '.join(norms))
+        if rank == 0:
+            with open(dumpfile, 'w') as f:
+                f.write("{%s}" % ', '.join(norms))
 
     return retval
 
@@ -273,7 +322,7 @@ def run_jit_backdoor(problem, **kwargs):
 
     @switchconfig(jit_backdoor=True)
     def _run_jit_backdoor():
-        return solver.forward(autotune=autotune)
+        return run_op(solver, 'forward', autotune=autotune)
 
     return _run_jit_backdoor()
 
