@@ -12,7 +12,7 @@ import numpy.ctypeslib as npct
 from codepy.jit import compile_from_string
 from codepy.toolchain import GCCToolchain
 
-from devito.archinfo import NVIDIAX, SKX, POWER8, POWER9
+from devito.archinfo import AMDGPUX, NVIDIAX, SKX, POWER8, POWER9
 from devito.exceptions import CompilationError
 from devito.logger import debug, warning, error
 from devito.parameters import configuration
@@ -351,7 +351,7 @@ class GNUCompiler(Compiler):
         super(GNUCompiler, self).__init__(*args, **kwargs)
 
         self.cflags += ['-march=native', '-Wno-unused-result', '-Wno-unused-variable',
-                        '-Wno-unused-but-set-variable', '--fast-math']
+                        '-Wno-unused-but-set-variable', '-ffast-math']
 
         language = kwargs.pop('language', configuration['language'])
         try:
@@ -383,11 +383,20 @@ class ClangCompiler(Compiler):
 
         if platform is NVIDIAX:
             self.cflags.remove('-std=c99')
-            # Clang has offloading support via OpenMP
+            # Add flags for OpenMP offloading
             if language in ['C', 'openmp']:
                 # TODO: Need a generic -march setup
                 # self.cflags += ['-Xopenmp-target', '-march=sm_37']
                 self.ldflags += ['-fopenmp', '-fopenmp-targets=nvptx64-nvidia-cuda']
+        elif platform is AMDGPUX:
+            self.cflags.remove('-std=c99')
+            # Add flags for OpenMP offloading
+            if language in ['C', 'openmp']:
+                self.ldflags += ['-target', 'x86_64-pc-linux-gnu']
+                self.ldflags += ['-fopenmp',
+                                 '-fopenmp-targets=amdgcn-amd-amdhs',
+                                 '-Xopenmp-target=amdgcn-amd-amdhsa']
+                self.ldflags += ['-march=%s' % platform.march]
         else:
             if platform in [POWER8, POWER9]:
                 # -march isn't supported on power architectures
@@ -404,6 +413,32 @@ class ClangCompiler(Compiler):
         self.MPICXX = 'mpicxx'
 
 
+class AOMPCompiler(Compiler):
+
+    """AMD's fork of Clang for OpenMP offloading on both AMD and NVidia cards."""
+
+    def __init__(self, *args, **kwargs):
+        super(AOMPCompiler, self).__init__(*args, **kwargs)
+
+        self.cflags += ['-Wno-unused-result', '-Wno-unused-variable', '-ffast-math']
+
+        platform = kwargs.pop('platform', configuration['platform'])
+
+        if platform in [NVIDIAX, AMDGPUX]:
+            self.cflags.remove('-std=c99')
+        elif platform in [POWER8, POWER9]:
+            # It doesn't make much sense to use AOMP on Power, but it should work
+            self.cflags += ['-mcpu=native']
+        else:
+            self.cflags += ['-march=native']
+
+    def __lookup_cmds__(self):
+        self.CC = 'aompcc'
+        self.CXX = 'aompcc'
+        self.MPICC = 'mpicc'
+        self.MPICXX = 'mpicxx'
+
+
 class PGICompiler(Compiler):
 
     def __init__(self, *args, **kwargs):
@@ -535,6 +570,7 @@ compiler_registry = {
     'gnu': GNUCompiler,
     'gcc': GNUCompiler,
     'clang': ClangCompiler,
+    'aomp': AOMPCompiler,
     'pgcc': PGICompiler,
     'pgi': PGICompiler,
     'osx': ClangCompiler,
