@@ -3,12 +3,11 @@ from itertools import product
 from operator import attrgetter
 
 from cached_property import cached_property
-from frozendict import frozendict
 from sympy import Max, Min
 
 from devito.data import CORE, OWNED, LEFT, CENTER, RIGHT
 from devito.ir.support import Scope
-from devito.tools import Tag, as_mapper, as_tuple, filter_ordered, flatten
+from devito.tools import Tag, as_mapper, as_tuple, filter_ordered, flatten, frozendict
 
 __all__ = ['HaloScheme', 'HaloSchemeEntry', 'HaloSchemeException']
 
@@ -77,13 +76,15 @@ class HaloScheme(object):
         # Track the IterationSpace offsets induced by SubDomains/SubDimensions.
         # These should be honored in the derivation of the `omapper`
         self._honored = {}
-        for i in ispace.intervals:
-            if not i.dim._defines & set(self.dimensions):
-                continue
-            elif i.dim.is_Sub and not i.dim.local:
-                ltk, _ = i.dim.thickness.left
-                rtk, _ = i.dim.thickness.right
-                self._honored[i.dim.root] = frozenset([(ltk, rtk)])
+        # SubDimensions are not necessarily included directly in
+        # ispace.dimensions and hence we need to first utilize the `_defines` method
+        dims = set().union(*[d._defines for d in ispace.dimensions
+                             if d._defines & self.dimensions])
+        subdims = [d for d in dims if d.is_Sub and not d.local]
+        for i in subdims:
+            ltk, _ = i.thickness.left
+            rtk, _ = i.thickness.right
+            self._honored[i.root] = frozenset([(ltk, rtk)])
         self._honored = frozendict(self._honored)
 
     def __repr__(self):
