@@ -15,13 +15,13 @@ from devito.ir.equations import LoweredEq
 from devito.ir.clusters import ClusterGroup, clusterize
 from devito.ir.iet import Callable, MetaCall, derive_parameters, iet_build, iet_lower_dims
 from devito.ir.stree import stree_build
+from devito.ir.equations.algorithms import lower_exprs
 from devito.operator.registry import operator_selector
 from devito.operator.profiling import create_profile
 from devito.mpi import MPI
 from devito.parameters import configuration
 from devito.passes import Graph
-from devito.symbolics import (estimate_cost, retrieve_functions, retrieve_indexed,
-                              uxreplace)
+from devito.symbolics import estimate_cost, retrieve_functions
 from devito.tools import (DAG, Signer, ReducerMap, as_tuple, flatten, filter_ordered,
                           filter_sorted, split, timed_pass, timed_region)
 from devito.types import Dimension, Eq
@@ -296,41 +296,9 @@ class Operator(Callable):
         # Scalarize tensor expressions
         expressions = [j for i in expressions for j in i._flatten]
 
-        # Indexification
-        # E.g., f(x - 2*h_x, y) -> f[xi + 2, yi + 4]  (assuming halo_size=4)
-        processed = []
-        for expr in expressions:
-            if expr.subdomain:
-                dimension_map = expr.subdomain.dimension_map
-            else:
-                dimension_map = {}
+        expressions = lower_exprs(expressions, **kwargs)
 
-            # Handle Functions (typical case)
-            mapper = {f: f.indexify(lshift=True, subs=dimension_map)
-                      for f in retrieve_functions(expr)}
-
-            # Handle Indexeds (from index notation)
-            for i in retrieve_indexed(expr):
-                f = i.function
-
-                # Introduce shifting to align with the computational domain
-                indices = [(a + o) for a, o in zip(i.indices, f._size_nodomain.left)]
-
-                # Apply substitutions, if necessary
-                if dimension_map:
-                    indices = [j.xreplace(dimension_map) for j in indices]
-
-                mapper[i] = f.indexed[indices]
-
-            subs = kwargs.get('subs')
-            if subs:
-                # Include the user-supplied substitutions, and use
-                # `xreplace` for constant folding
-                processed.append(expr.xreplace({**mapper, **subs}))
-            else:
-                processed.append(uxreplace(expr, mapper))
-
-        processed = cls._specialize_exprs(processed)
+        processed = cls._specialize_exprs(expressions)
 
         return processed
 
@@ -970,6 +938,8 @@ def parse_kwargs(**kwargs):
     options.setdefault('min-storage', opt_options.get('min-storage', False))
     options.setdefault('cire-repeats-inv', opt_options.get('cire-repeats-inv', None))
     options.setdefault('cire-repeats-sops', opt_options.get('cire-repeats-sops', None))
+    options.setdefault('cire-mincost-inv', opt_options.get('cire-mincost-inv', None))
+    options.setdefault('cire-mincost-sops', opt_options.get('cire-mincost-sops', None))
     options.setdefault('openmp', openmp)
     options.setdefault('mpi', configuration['mpi'])
     kwargs['options'] = options
