@@ -10,12 +10,13 @@ import sympy
 from sympy.core.assumptions import _assume_rules
 from cached_property import cached_property
 from cgen import Struct, Value
-from frozendict import frozendict
 
 from devito.data import default_allocator
 from devito.finite_differences import Evaluable
 from devito.parameters import configuration
-from devito.tools import Pickable, ctypes_to_cstr, dtype_to_cstr, dtype_to_ctype
+from devito.symbolics import aligned_indices
+from devito.tools import (Pickable, ctypes_to_cstr, dtype_to_cstr, dtype_to_ctype,
+                          frozendict)
 from devito.types.args import ArgProvider
 from devito.types.caching import Cached
 from devito.types.utils import DimensionTuple
@@ -724,9 +725,25 @@ class AbstractFunction(sympy.Function, Basic, Cached, Pickable, Evaluable):
         """Tuple of Dimensions representing the object indices."""
         return self._dimensions
 
+    @property
+    def _eval_deriv(self):
+        return self
+
+    @cached_property
+    def _is_on_grid(self):
+        """
+        Check whether the object is on the grid or need averaging.
+        For example, if the original non-staggered function is f(x)
+        then f(x) is on the grid and f(x + h_x/2) is off the grid.
+        """
+        return all([aligned_indices(i, j, d.spacing) for i, j, d in
+                    zip(self.indices, self.indices_ref, self.dimensions)])
+
     @property
     def evaluate(self):
         # Average values if at a location not on the Function's grid
+        if self._is_on_grid:
+            return self
         weight = 1.0
         avg_list = [self]
         is_averaged = False
@@ -933,7 +950,7 @@ class AbstractFunction(sympy.Function, Basic, Cached, Pickable, Evaluable):
         # Indices after substitutions
         indices = [(a - o + f).xreplace(s) for a, o, f, s in
                    zip(self.args, self.origin, shift, subs)]
-
+        indices = [i.xreplace({k: int(k) for k in i.atoms(sympy.Float)}) for i in indices]
         return self.indexed[indices]
 
     def __getitem__(self, index):
