@@ -1,11 +1,12 @@
 import numpy as np
+from argparse import ArgumentParser
 
-from devito import error
+from devito import error, configuration, warning
 from devito.tools import Pickable
 
 from .source import *
 
-__all__ = ['AcquisitionGeometry', 'setup_geometry']
+__all__ = ['AcquisitionGeometry', 'setup_geometry', 'seismic_args']
 
 
 def setup_geometry(model, tn):
@@ -13,7 +14,7 @@ def setup_geometry(model, tn):
     src_coordinates = np.empty((1, model.dim))
     src_coordinates[0, :] = np.array(model.domain_size) * .5
     if model.dim > 1:
-        src_coordinates[0, -1] = model.origin[-1] + 2 * model.spacing[-1]
+        src_coordinates[0, -1] = model.origin[-1] + model.spacing[-1]
 
     rec_coordinates = setup_rec_coords(model)
 
@@ -141,9 +142,24 @@ class AcquisitionGeometry(Pickable):
                         time_range=self.time_axis, npoint=self.nrec,
                         coordinates=self.rec_positions)
 
+    @property
+    def adj_src(self):
+        if self.src_type is None:
+            warning("No surce type defined, returning uninitiallized (zero) shot record")
+            return self.rec
+        adj_src = sources[self.src_type](name='rec', grid=self.grid, f0=self.f0,
+                                         time_range=self.time_axis, npoint=self.nrec,
+                                         coordinates=self.rec_positions,
+                                         t0=self._t0w, a=self._a)
+        # Revert time axis to have a proper shot record and not compute on zeros
+        for i in range(self.nrec):
+            adj_src.data[:, i] = adj_src.wavelet[::-1]
+        return adj_src
+
     @property
     def src(self):
         if self.src_type is None:
+            warning("No surce type defined, returning uninistiallized (zero) source")
             return PointSource(name='src', grid=self.grid,
                                time_range=self.time_axis, npoint=self.nsrc,
                                coordinates=self.src_positions)
@@ -158,3 +174,36 @@ class AcquisitionGeometry(Pickable):
 
 
 sources = {'Wavelet': WaveletSource, 'Ricker': RickerSource, 'Gabor': GaborSource}
+
+
+def seismic_args(description):
+    """
+    Command line options for the seismic examples
+    """
+    parser = ArgumentParser(description=description)
+    parser.add_argument("-nd", dest="ndim", default=3, type=int,
+                        help="Number of dimensions")
+    parser.add_argument("-d", "--shape", default=(51, 51, 51), type=int, nargs="+",
+                        help="Number of grid points along each axis")
+    parser.add_argument('-f', '--full', default=False, action='store_true',
+                        help="Execute all operators and store forward wavefield")
+    parser.add_argument("-so", "--space_order", default=4,
+                        type=int, help="Space order of the simulation")
+    parser.add_argument("--nbl", default=40,
+                        type=int, help="Number of boundary layers around the domain")
+    parser.add_argument("-k", dest="kernel", default='OT2',
+                        choices=['OT2', 'OT4', 'centered', 'staggered'],
+                        help="Choice of finite-difference kernel")
+    parser.add_argument("--constant", default=False, action='store_true',
+                        help="Constant velocity model, default is a two layer model")
+    parser.add_argument("--checkpointing", default=False, action='store_true',
+                        help="Constant velocity model, default is a two layer model")
+    parser.add_argument("-opt", default="advanced",
+                        choices=configuration._accepted['opt'],
+                        help="Performance optimization level")
+    parser.add_argument('-a', '--autotune', default='off',
+                        choices=(configuration._accepted['autotuning']),
+                        help="Operator auto-tuning mode")
+    parser.add_argument('--noazimuth', dest='azi', default=False, action='store_true',
+                        help="Whether or not to use an azimuth angle")
+    return parser.parse_args()
